import typing

# See
# http://peak.telecommunity.com/DevCenter/setuptools#namespace-packages
try:
    __import__("pkg_resources").declare_namespace(__name__)
except ImportError:
    from pkgutil import extend_path

    __path__: typing.List[str] = extend_path(  # type: ignore[no-redef]
        __path__, __name__
    )



"""
"""

from wheezy.template.engine import Engine
from wheezy.template.ext.code import CodeExtension
from wheezy.template.ext.core import CoreExtension
from wheezy.template.loader import DictLoader, FileLoader, PreprocessLoader
from wheezy.template.preprocessor import Preprocessor

__all__ = (
    "Engine",
    "CodeExtension",
    "CoreExtension",
    "DictLoader",
    "FileLoader",
    "PreprocessLoader",
    "Preprocessor",
)

__version__ = "3.2.1"



import typing
from abc import abstractmethod

from wheezy.template.comp import List, Tuple

Token = Tuple[int, str, str]


class Builder:
    lineno: int

    @abstractmethod
    def start_block(self) -> None:
        ...  # pragma: nocover

    @abstractmethod
    def end_block(self) -> None:
        ...  # pragma: nocover

    @abstractmethod
    def add(self, lineno: int, code: str) -> None:
        ...  # pragma: nocover

    @abstractmethod
    def build_block(self, nodes: typing.Iterable[Token]) -> None:
        ...  # pragma: nocover

    @abstractmethod
    def build_token(
        self,
        lineno: int,
        token: str,
        value: typing.Union[str, typing.Iterable[Token]],
    ) -> None:
        ...  # pragma: nocover


Tokenizer = typing.Callable[[typing.Match], Token]
LexerRule = Tuple[typing.Pattern, Tokenizer]
PreProcessorRule = typing.Callable[[str], str]
PostProcessorRule = typing.Callable[[List[Token]], str]
BuilderRule = typing.Callable[
    [
        Builder,
        int,
        str,
        typing.Union[str, List[str], typing.Iterable[Token]],
    ],
    bool,
]
ParserRule = typing.Callable[[str], typing.Union[str, List[str]]]


class ParserConfig:
    end_tokens: List[str]
    continue_tokens: List[str]
    compound_tokens: List[str]
    out_tokens: List[str]


RenderTemplate = typing.Callable[
    [
        typing.Mapping[str, typing.Any],
        typing.Mapping[str, typing.Any],
        typing.Mapping[str, typing.Any],
    ],
    str,
]


class SupportsRender:
    @abstractmethod
    def render(self, ctx: typing.Mapping[str, typing.Any]) -> str:
        ...  # pragma: nocover


TemplateClass = typing.Callable[[str, RenderTemplate], SupportsRender]


class Loader:
    @abstractmethod
    def list_names(self) -> Tuple[str, ...]:
        ...  # pragma: nocover

    @abstractmethod
    def load(self, name: str) -> typing.Optional[str]:
        ...  # pragma: nocover



import typing

from wheezy.template.typing import Builder, BuilderRule, Token


def builder_scan(
    extensions: typing.List[typing.Any],
) -> typing.Mapping[str, typing.Any]:
    builder_rules: typing.Dict[str, typing.List[BuilderRule]] = {}
    for extension in extensions:
        if hasattr(extension, "builder_rules"):
            rules = extension.builder_rules
            for token, builder in rules:
                builder_rules.setdefault(token, []).append(builder)
    return {"builder_rules": builder_rules}


class BlockBuilder(Builder):
    __slots__ = ("rules", "indent", "lineno", "buf")

    def __init__(
        self,
        rules: typing.Dict[str, typing.List[BuilderRule]],
        indent: str = "",
        lineno: int = 0,
    ) -> None:
        self.rules = rules
        self.indent = indent
        self.lineno = lineno
        self.buf: typing.List[str] = []

    def start_block(self) -> None:
        self.indent += "    "

    def end_block(self) -> None:
        if len(self.indent) < 4:
            raise SyntaxError("Unexpected end of block.")
        self.indent = self.indent[:-4]

    def add(self, lineno: int, code: str) -> None:
        if lineno < self.lineno:
            raise SyntaxError(
                "Inconsistence at %s : %s" % (self.lineno, lineno)
            )
        if lineno == self.lineno:
            line = self.buf[-1]
            if code:
                if line:
                    if line[-1:] == ":":
                        self.buf[-1] = line + code
                    else:
                        self.buf[-1] = line + "; " + code
                else:
                    self.buf[-1] = self.indent + code
        else:
            pad = lineno - self.lineno - 1
            if pad > 0:
                self.buf.extend([""] * pad)
            if code:
                self.buf.append(self.indent + code)
            else:
                self.buf.append("")
        self.lineno = lineno + code.count("\n")

    def build_block(self, nodes: typing.Iterable[Token]) -> None:
        for lineno, token, value in nodes:
            self.build_token(lineno, token, value)

    def build_token(
        self,
        lineno: int,
        token: str,
        value: typing.Union[str, typing.Iterable[Token]],
    ) -> None:
        if token in self.rules:
            for rule in self.rules[token]:
                if rule(self, lineno, token, value):
                    break
        else:
            raise SyntaxError(
                'No rule to build "%s" token at line %d.' % (token, lineno)
            )

    def to_string(self) -> str:
        return "\n".join(self.buf)


class SourceBuilder(object):
    __slots__ = ("rules", "lineno")

    def __init__(
        self,
        builder_rules: typing.Dict[str, typing.List[BuilderRule]],
        builder_offset: int = 2,
        **ignore: typing.Any
    ) -> None:
        self.rules = builder_rules
        self.lineno = 0 - builder_offset

    def build_source(self, nodes: typing.Iterable[Token]) -> str:
        builder = BlockBuilder(self.rules)
        builder.build_block(nodes)
        return builder.to_string()

    def build_render(self, nodes: typing.Iterable[Token]) -> str:
        builder = BlockBuilder(self.rules, lineno=self.lineno)
        builder.add(
            self.lineno + 1, "def render(ctx, local_defs, super_defs):"
        )
        builder.start_block()
        builder.build_token(self.lineno + 2, "render", nodes)
        return builder.to_string()

    def build_module(self, nodes: typing.Iterable[Token]) -> str:
        builder = BlockBuilder(self.rules, lineno=self.lineno)
        builder.add(self.lineno + 1, "local_defs = {}; super_defs = {}")
        builder.build_token(self.lineno + 2, "module", nodes)
        return builder.to_string()



#!/usr/bin/env python

import multiprocessing
import os
import re

from setuptools import setup  # type: ignore[import]

extra = {}
try:
    from Cython.Build import cythonize  # type: ignore[import]

    p = os.path.join("src", "wheezy", "template")
    extra["ext_modules"] = cythonize(
        [os.path.join(p, "*.py"), os.path.join(p, "ext", "*.py")],
        exclude=[
            os.path.join(p, "__init__.py"),
            os.path.join(p, "ext", "__init__.py"),
        ],
        # https://github.com/cython/cython/issues/3262
        nthreads=0 if multiprocessing.get_start_method() == "spawn" else 2,
        compiler_directives={"language_level": 3},
        quiet=True,
    )
except ImportError:
    pass

README = open("README.md").read()
VERSION = (
    re.search(  # type: ignore
        r'__version__ = "(.+)"',
        open("src/wheezy/template/__init__.py").read(),
    )
    .group(1)
    .strip()
)

setup(
    name="wheezy.template",
    version=VERSION,
    python_requires=">=3.8",
    description="A lightweight template library",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/akornatskyy/wheezy.template",
    author="Andriy Kornatskyy",
    author_email="andriy.kornatskyy@live.com",
    license="MIT",
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Widget Sets",
        "Topic :: Text Processing :: Markup :: HTML",
    ],
    keywords="html markup template preprocessor",
    packages=["wheezy", "wheezy.template", "wheezy.template.ext"],
    package_data={"wheezy.template": ["py.typed"]},
    package_dir={"": "src"},
    namespace_packages=["wheezy"],
    zip_safe=False,
    install_requires=[],
    entry_points={
        "console_scripts": ["wheezy.template=wheezy.template.console:main"]
    },
    platforms="any",
    **extra
)
